﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using BraStor_WebRole.Model;
using BraStor_WebRole.Dal;
using System.Web.Security;
using System.Net.Mail;
using System.Net;


namespace BraStor_WebRole.Bll
{
    public class BllIdeas
    {
        /// <summary>
        /// Obtiene una lista de todos los temas activos
        /// </summary>
        /// <returns></returns>
        public List<tbl_temas> getActiveThemes()
        {
            DalThemes dThemes = new DalThemes();
            try
            {
                List<tbl_temas> lstTemas = new List<tbl_temas>();
                lstTemas = dThemes.getActiveThemes();

                return lstTemas;
            }
            catch (Exception ex)
            {
                throw (ex);
            }
        }

        /// <summary>
        /// Obtiene una lista de todos los temas activos
        /// </summary>
        /// <returns></returns>
        public List<tbl_temas> getAllThemes()
        {
            DalThemes dThemes = new DalThemes();
            try
            {
                List<tbl_temas> lstTemas = new List<tbl_temas>();
                lstTemas = dThemes.getAllThemes();

                lstTemas.Insert(0,new tbl_temas());

                return lstTemas;
            }
            catch (Exception ex)
            {
                throw (ex);
            }
        }

        /// <summary>
        /// Obtiene una lista de todos los usuarios que no se encuentran bloqueados
        /// </summary>
        /// <returns></returns>
        public List<vw_Users> getActiveUsers()
        {
            DalUsers dUsers = new DalUsers();
            try
            {
                List<vw_Users> lstUsers = new List<vw_Users>();
                lstUsers = dUsers.getListActiveUsers();

                return lstUsers;
            }
            catch (Exception ex)
            {
                throw (ex);
            }
        }

        /// <summary>
        /// Obtiene todos los posibles estados de una idea
        /// </summary>
        /// <returns></returns>
        public List<tbl_sys_estadoIdea> getAllStatus()
        {
            DalEstado dEstado = new DalEstado();
            try
            {
                List<tbl_sys_estadoIdea> lstStatus = new List<tbl_sys_estadoIdea>();
                lstStatus = dEstado.getAllStatus();

                lstStatus.Insert(0,new tbl_sys_estadoIdea());

                return lstStatus;
            }
            catch (Exception ex)
            {
                throw (ex);
            }
        }

        /// <summary>
        /// Obtiene todas las posibles razones de archivo
        /// </summary>
        /// <returns></returns>
        public List<tbl_sys_razonArchivo> getAllArchiveReason()
        {
            DalArchive dArchive = new DalArchive();
            try
            {
                List<tbl_sys_razonArchivo> lstArchives = new List<tbl_sys_razonArchivo>();
                lstArchives = dArchive.getAllArchiveReason();

                return lstArchives;
            }
            catch (Exception ex)
            {
                throw (ex);
            }
        }


        /// <summary>
        /// Retorna un listado de ideas en base a unos criterios establecidos.
        /// </summary>
        /// <param name="ideaCriteria"></param>
        /// <param name="bArchivo"></param>
        /// <returns></returns>
        public List<vw_Ideas> getListIdeas(vw_Ideas ideaCriteria, int int_archivo)
        {
            DalIdeas dIdeas = new DalIdeas();
            try
            {
                List<vw_Ideas> lstIdeas = new List<vw_Ideas>();

                lstIdeas = dIdeas.getListIdeas(ideaCriteria,int_archivo);

                return lstIdeas;
            }
            catch (Exception ex)
            {
                throw (ex);
            }
        }


        /// <summary>
        /// Obtiene un ranking de ideas
        /// </summary>
        /// <param name="rankingCriteria"></param>
        /// <param name="int_archivo"></param>
        /// <returns></returns>
        public List<vw_rankingIdeas> getRankingIdeas(string idTema, int int_archivo)
        {
            DalIdeas dIdeas = new DalIdeas();
            try
            {
                List<vw_rankingIdeas> lstRankingIdeas = new List<vw_rankingIdeas>();

                lstRankingIdeas = dIdeas.getRankingIdeas(idTema, int_archivo);

                return lstRankingIdeas;
            }
            catch (Exception ex)
            {
                throw (ex);
            }
        }

        /// <summary>
        /// Obtiene los datos de un usuario a partir de su identificador.
        /// </summary>
        /// <param name="idUser"></param>
        /// <returns></returns>
        public vw_Users getUserDetail(string idUser)
        {
            DalUsers dUser = new DalUsers();
            try
            {
               vw_Users userDetail =  dUser.getDetailUser(idUser);

               return userDetail;
            }
            catch (Exception ex)
            {
                return null;
            }
        }

        /// <summary>
        /// Busca el Detalle de un usuario en base a su nombre de usuarios
        /// </summary>
        /// <param name="idUser"></param>
        /// <returns></returns>
        public vw_Users getUserDetailByName(string userName)
        {
            DalUsers dUser = new DalUsers();
            try
            {
                vw_Users userDetail = dUser.getDetailUserByName(userName);

                return userDetail;
            }
            catch (Exception)
            {
                return null;
            }
        }

        /// <summary>
        /// Inserta una nueva idea
        /// </summary>
        /// <param name="newIdea"></param>
        internal void createIdea(tbl_ideas newIdea)
        {
            DalUsers dUser = new DalUsers();
            DalIdeas dIdeas = new DalIdeas();
            try
            {
                //Valores por defecto de la nueva Idea
                newIdea.dat_fechaCreacion = DateTime.UtcNow;
                newIdea.bol_archivada = false;
                newIdea.epk_lng_idEstado = 1;       //El código 1 corresponde a una Idea Abierta
                //Busca el ID del usuario identificado en el sistema
                vw_Users userCriteria = new vw_Users();
                userCriteria.UserName = HttpContext.Current.User.Identity.Name;
                List<vw_Users> lstUser = dUser.getListUsers(userCriteria);
                if (lstUser != null && lstUser.Count() > 0)
                    newIdea.epk_uid_propietario = lstUser[0].UserId;
                else
                    throw new Exception("No hay ningún usuario conectado");
                
                //Llama a la función que realmente insertará los datos
                dIdeas.insertNewIdea(newIdea);

                //Después de haber sido dada de alta la ideea hay que enviar un corre a todos los usuarios
                //Message
                MailMessage eMail = new MailMessage();

                //From
                string str_nameFrom = String.IsNullOrWhiteSpace(lstUser[0].str_nombre + " " + lstUser[0].str_apellidos) ? lstUser[0].UserName : lstUser[0].str_nombre + " " + lstUser[0].str_apellidos;
                eMail.From = new MailAddress(lstUser[0].Email, str_nameFrom);
                //To
                MailAddressCollection toCol = new MailAddressCollection();
                for (int iu = 0; iu < newIdea.tbl_rel_ideausuario.Count; iu++)
                {
                    Guid idUser = newIdea.tbl_rel_ideausuario.ElementAt(iu).epk_uid_usuario;
                    vw_Users userTo = dUser.getDetailUser(idUser.ToString());
                    string str_nameTo = String.IsNullOrWhiteSpace(userTo.str_nombre + " " + userTo.str_apellidos)?userTo.UserName:userTo.str_nombre + " " + userTo.str_apellidos;

                    eMail.To.Add(new MailAddress(userTo.Email, str_nameTo));
                }

                eMail.Subject = "[IDEAS] Nueva Idea";
                eMail.IsBodyHtml = true;
                eMail.Body = "<b>NUEVA IDEA</b><br/><br/><p>Una nueva idea ha sido introducida en el sistema.<br/>Ya puede acceder al sistema para realizar sus comentarios.<br/><br/></p>";
                eMail.Body = eMail.Body +"<b>Título: </b> " + newIdea.str_titulo + "<br/>";
                eMail.Body = eMail.Body +"<b>Fecha límite: </b> " + newIdea.dat_fechalimite.ToShortDateString() + "<br/>";
                eMail.Body = eMail.Body +"<br/>Muchas gracias por su colaboración<br/><br/>";
                eMail.Body = eMail.Body + str_nameFrom;

                SmtpClient smtp = new SmtpClient();
                smtp.Host = "smtp.gmail.com";
                smtp.EnableSsl = true;
                smtp.Port = 587;

                 NetworkCredential credencial = new NetworkCredential("vegamar76@gmail.com", "vgm123!!");
                 smtp.Credentials = credencial;

                smtp.Send(eMail);

            }
            catch (Exception ex)
            {
                throw (ex);
            }
        }

        /// <summary>
        /// Modifica los datos de una idea
        /// </summary>
        /// <param name="IdeaDetail"></param>
        internal void updateIdea(tbl_ideas ideaDetail, Boolean bol_eMail)
        {
            DalUsers dUser = new DalUsers();
            DalIdeas dIdeas = new DalIdeas();
            try
            {
                //Busca el ID del usuario identificado en el sistema
                vw_Users userCriteria = new vw_Users();
                userCriteria.UserName = HttpContext.Current.User.Identity.Name;
                List<vw_Users> lstUser = dUser.getListUsers(userCriteria);

                //Llama a la función que realmente insertará los datos
                dIdeas.updateIdea(ideaDetail);

                if (bol_eMail)
                {
                    //Después de haber sido dada de alta la ideea hay que enviar un corre a todos los usuarios
                    //Message
                    MailMessage eMail = new MailMessage();

                    //From
                    string str_nameFrom = String.IsNullOrWhiteSpace(lstUser[0].str_nombre + " " + lstUser[0].str_apellidos) ? lstUser[0].UserName : lstUser[0].str_nombre + " " + lstUser[0].str_apellidos;
                    eMail.From = new MailAddress(lstUser[0].Email, str_nameFrom);
                    //To
                    MailAddressCollection toCol = new MailAddressCollection();
                    for (int iu = 0; iu < ideaDetail.tbl_rel_ideausuario.Count; iu++)
                    {
                        Guid idUser = ideaDetail.tbl_rel_ideausuario.ElementAt(iu).epk_uid_usuario;
                        vw_Users userTo = dUser.getDetailUser(idUser.ToString());
                        string str_nameTo = String.IsNullOrWhiteSpace(userTo.str_nombre + " " + userTo.str_apellidos) ? userTo.UserName : userTo.str_nombre + " " + userTo.str_apellidos;

                        eMail.To.Add(new MailAddress(userTo.Email, str_nameTo));
                    }

                    eMail.Subject = "[IDEAS] Actualización de Idea";
                    eMail.IsBodyHtml = true;
                    eMail.Body = "<b>ACTUALIZACIÓN DE IDEA</b><br/><br/><p>'Atención!. La siguiente idea ha sufrido modificaciones<br/>Ya puede acceder al sistema para realizar sus comentarios.<br/><br/></p>";
                    eMail.Body = eMail.Body + "<b>Título: </b> " + ideaDetail.str_titulo + "<br/>";
                    eMail.Body = eMail.Body + "<b>Fecha límite: </b> " + ideaDetail.dat_fechalimite.ToShortDateString() + "<br/>";
                    eMail.Body = eMail.Body + "<br/>Muchas gracias por su colaboración<br/><br/>";
                    eMail.Body = eMail.Body + str_nameFrom;

                    SmtpClient smtp = new SmtpClient();
                    smtp.Host = "smtp.gmail.com";
                    smtp.EnableSsl = true;
                    smtp.Port = 587;

                    NetworkCredential credencial = new NetworkCredential("vegamar76@gmail.com", "vgm123!!");
                    smtp.Credentials = credencial;

                    smtp.Send(eMail);
                }

            }
            catch (Exception ex)
            {
                throw (ex);
            }

        }


        /// <summary>
        /// Recoge los datos de una idea a partir de su clave primaria
        /// </summary>
        /// <param name="idIdea"></param>
        /// <returns></returns>
        internal tbl_ideas getIdeaDetail(string idIdea)
        {
            DalIdeas dIdeas = new Dal.DalIdeas();
            try
            {
                tbl_ideas ideaDetail = new tbl_ideas();

                ideaDetail = dIdeas.getIdeaDetail(idIdea);

                return ideaDetail;
            }
            catch (Exception ex)
            {
                throw (ex);
            }
        }

        /// <summary>
        /// Obtiene todos los comentarios asociados a una misma idea
        /// </summary>
        /// <param name="idIdea"></param>
        /// <returns></returns>
        public List<vw_Comentarios> getComentariosIdea(string idIdea)
        {
            DalComentarios dComentarios = new DalComentarios();
            try
            {
                List<vw_Comentarios> lstComments = new List<vw_Comentarios>();

                lstComments = dComentarios.getComentariosIdea(idIdea);

                return lstComments;
            }
            catch (Exception ex)
            {
                throw (ex);
            }
        }


        /// <summary>
        /// Añade un nuevo comentario a una idea
        /// </summary>
        /// <param name="guid"></param>
        /// <param name="p"></param>
        /// <param name="str_comments"></param>
        internal void addComments(Guid guid, string userName, string str_comments)
        {
            DalComentarios dComments = new DalComentarios();
            DalUsers dUser = new DalUsers();
            try
            {
                //Recoge el ID del usuario
                //Busca el ID del usuario identificado en el sistema
                vw_Users userCriteria = new vw_Users();
                userCriteria.UserName = userName;
                List<vw_Users> lstUser = dUser.getListUsers(userCriteria);
                Guid userId = lstUser[0].UserId;

                dComments.addCommentIdea(guid, userId, str_comments);

            }
            catch (Exception ex)
            {
                throw (ex);
            }
        }

        /// <summary>
        /// Obtiene la valoración media de una idea
        /// </summary>
        /// <param name="idIdea"></param>
        /// <returns></returns>
        public Double getMeanValueIdea(string idIdea)
        {
            DalValoracion dValoration = new DalValoracion();

            try
            {
                return dValoration.getAverageIdea(idIdea);
            }
            catch (Exception)
            {
                return 0;
            }
        }

        /// <summary>
        /// Obtiene la nota que un usuario ha dado a una idea
        /// </summary>
        /// <param name="idIdea"></param>
        /// <param name="userName"></param>
        /// <returns></returns>
        public Double getNoteOfIdea_User(string idIdea, string userName)
        {
            DalValoracion dValoration = new DalValoracion();
            DalUsers dUser = new DalUsers();

            try
            {
                //Busca el ID del usuario identificado en el sistema
                vw_Users userCriteria = new vw_Users();
                userCriteria.UserName = userName;
                List<vw_Users> lstUser = dUser.getListUsers(userCriteria);
                string str_userId = lstUser[0].UserId.ToString();

                return dValoration.getNoteIdeaUser(idIdea, str_userId);

            }
            catch (Exception)
            {
                return 0;
            }
        }

        /// <summary>
        /// Valoración de una idea por un usuario
        /// </summary>
        /// <param name="guid"></param>
        /// <param name="p"></param>
        /// <param name="int_nota"></param>
        internal void setValueIdea(Guid guid, string userName, int int_nota)
        {
            DalValoracion dValoracion = new DalValoracion();
            DalUsers dUser = new DalUsers();
            try
            {
                //Recoge el ID del usuario
                //Busca el ID del usuario identificado en el sistema
                vw_Users userCriteria = new vw_Users();
                userCriteria.UserName = userName;
                List<vw_Users> lstUser = dUser.getListUsers(userCriteria);
                Guid userId = lstUser[0].UserId;

                dValoracion.setValorarIdea(guid, userId, int_nota);
            }
            catch (Exception ex)
            {
                throw (ex);
            }
        }


        /// <summary>
        /// Archiva o reabre una en función de estado que tenga
        /// </summary>
        /// <param name="guid"></param>
        internal void archivarIdea(Guid uid_idea, int id_reason)
        {
            DalIdeas dIdeas = new DalIdeas();

            try
            {
                //Cuando se archiva una idea se cambia su estado a archivado
                int id_estado = 3;

                dIdeas.updateIdeaReasonArchive(uid_idea, id_reason, id_estado);
            }
            catch (Exception ex)
            {
                throw (ex);
            }
        }

    }
}