﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using BraStor_WebRole.Dal;
using BraStor_WebRole.Model;
using System.Web.Security;

namespace BraStor_WebRole.Bll
{
    public class BllUsers
    {

        /// <summary>
        /// Añade la personalización de usuario a la nueva cuenta de usuario generada
        /// </summary>
        /// <param name="newPersUser"></param>
        /// <param name="str_Role"></param>
        public void createUser(tbl_personalizacionUsuarios newPersUser, string str_Role)
        {
            try
            {
                DalUsers dUserPers = new DalUsers();

                //Asignar role al usuario
                if (Roles.RoleExists(str_Role))
                {
                    MembershipUser mshipUser =  Membership.GetUser(newPersUser.pk_uid_id);
                    string[] str_users = {mshipUser.UserName};

                    Roles.AddUsersToRole(str_users, str_Role);
                }

                //Inserta la personalización de los usuarios
                dUserPers.newPersonalization(newPersUser);

            }
            catch (Exception ex)
            {
                throw (ex);
            }
        }

        /// <summary>
        /// Obtiene un listado de todos los usuarios de la aplicación
        /// </summary>
        /// <returns></returns>
        public List<vw_Users> getAllUsers()
        {
            try
            {
                DalUsers dUsers = new DalUsers();

                vw_Users userCriteria = new vw_Users();

                List<vw_Users> lstUsers = dUsers.getListUsers(userCriteria);

                return lstUsers;
            }
            catch (Exception ex)
            {
                throw(ex);
            }
        }

        /// <summary>
        /// Obtiene el detalle de un usuario
        /// </summary>
        /// <param name="userUid"></param>
        /// <returns></returns>
        public vw_Users getUserDetail(string userUid)
        {
            try
            {
                DalUsers dUser = new DalUsers();

                vw_Users userDetail = dUser.getDetailUser(userUid);

                if (userDetail.uid_organizacion == null)
                    userDetail.uid_organizacion = new Guid();

                return userDetail;
            }
            catch (Exception ex)
            {
                throw (ex);
            }
        }

        /// <summary>
        /// Realiza la llamada a la actualización de los datos de un usuario
        /// </summary>
        /// <param name="userDetail"></param>
        public void UpdateDetail(vw_Users userDetail)
        {
            DalUsers dUsers = new DalUsers();
            try
            {
                dUsers.updateUser(userDetail);

            }
            catch (Exception ex)
            {
                throw (ex);
            }
        }

        /// <summary>
        /// Desbloquea un usuario bloqueado con anterioridad
        /// </summary>
        /// <param name="userId"></param>
        public void UnlockUser(Guid userId)
        {
            try
            {
                MembershipUser mbs_user = Membership.GetUser(userId);
                if (mbs_user.IsLockedOut) 
                    mbs_user.UnlockUser();

            }
            catch (Exception ex)
            {
                throw (ex);
            }

        }

          /// <summary>
        /// Obtiene todas las organizaciones existentes en la base de dato
        /// </summary>
        /// <returns></returns>
        public List<tbl_organizaciones> getAllOrganizations()
        {
            try
            {
                DalOrganizaciones dOrganization = new DalOrganizaciones();

                List<tbl_organizaciones> lstOrganizations = dOrganization.getListOrganizations(new tbl_organizaciones());

                //Añade una organización en blanco
                lstOrganizations.Insert(0, new tbl_organizaciones());

                return lstOrganizations;
            }
            catch (Exception)
            {
                return new List<tbl_organizaciones>();
            }

        }

        /// <summary>
        /// Devuelve un listado de todos los roles existentes en la aplicación
        /// </summary>
        /// <returns></returns>
        public List<String> getRoles()
        {
            string[] roles = Roles.GetAllRoles();
            
            return roles.ToList<String>();
        }
    }
}