﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using BraStor_WebRole.Model;
using System.Configuration;
using System.Data.Objects;
using LinqKit;

namespace BraStor_WebRole.Dal
{
    public class DalIdeas
    {
        UOC_IDEASEntities ideasBD;

        public DalIdeas()
        {
            string str_connection = ConfigurationManager.ConnectionStrings["UOC_IDEASEntities"].ConnectionString;
            ideasBD = new UOC_IDEASEntities(str_connection);
        }

        /// <summary>
        /// Inserta una nueva idea en la base de datos
        /// </summary>
        /// <param name="newIdea"></param>
        public void insertNewIdea(tbl_ideas newIdea)
        {
            try
            {
                //Pk
                Guid guid_idea = Guid.NewGuid();
                newIdea.pk_uid_id = guid_idea;

                //Pks para los temas relacionados
                for (int it = 0; it < newIdea.tbl_rel_ideatema.Count; it++)
                {
                    newIdea.tbl_rel_ideatema.ElementAt(it).epk_uid_idea = guid_idea;
                    newIdea.tbl_rel_ideatema.ElementAt(it).pk_uid_id = Guid.NewGuid();
                }

                //Pks para los usuarios relacionados
                for (int it = 0; it < newIdea.tbl_rel_ideausuario.Count; it++)
                {
                    newIdea.tbl_rel_ideausuario.ElementAt(it).epk_uid_idea = guid_idea;
                    newIdea.tbl_rel_ideausuario.ElementAt(it).pk_uid_id = Guid.NewGuid();
                }


                ideasBD.AddTotbl_ideas(newIdea);
                ideasBD.SaveChanges(SaveOptions.AcceptAllChangesAfterSave);
            }
            catch (Exception ex)
            {
                throw (ex);
            }
        }

        /// <summary>
        /// Busca todas las ideas existentes en la base de datos conforme a los criterios establecidos
        /// </summary>
        /// <param name="ideaCriteria"></param>
        /// <param name="bArchivo"></param>
        /// <returns></returns>
        internal List<vw_Ideas> getListIdeas(vw_Ideas ideaCriteria, int int_archivo)
        {
            try
            {
                var predicate = PredicateBuilder.True<vw_Ideas>();
                var predicateTitle = PredicateBuilder.False<vw_Ideas>();
                var predicateDescription = PredicateBuilder.False<vw_Ideas>();

                //Si se ha introducido algún criterio de busqueda. Añade criterios al predicado
                if (ideaCriteria != null)
                {
                    if (ideaCriteria.pk_uid_idTema != null && ideaCriteria.pk_uid_idTema != Guid.Empty)
                        predicate = predicate.And(idea => idea.pk_uid_idTema == ideaCriteria.pk_uid_idTema);

                    if (ideaCriteria.epk_lng_idEstado > 0)
                        predicate = predicate.And(idea => idea.epk_lng_idEstado == ideaCriteria.epk_lng_idEstado);

                    if (int_archivo >= 0)
                        predicate = predicate.And(idea => idea.bol_archivada == Convert.ToBoolean(int_archivo));

                    char[] separator = { ' ' };
                    if (!String.IsNullOrEmpty(ideaCriteria.str_titulo))
                    {
                        string[] str_criteria = ideaCriteria.str_titulo.Split(separator, StringSplitOptions.RemoveEmptyEntries);
                        foreach (string str_word in str_criteria)
                        {
                            string temp = str_word;
                            predicateTitle = predicateTitle.Or(idea => idea.str_titulo.Contains(temp));
                        }

                    }

                    if (!String.IsNullOrEmpty(ideaCriteria.str_descripcion))
                    {
                        string[] str_criteria = ideaCriteria.str_descripcion.Split(separator, StringSplitOptions.RemoveEmptyEntries);
                        foreach (string str_word in str_criteria)
                        {
                            string temp = str_word;
                            if (!String.IsNullOrWhiteSpace(str_word))
                                predicateTitle = predicateTitle.Or(idea => idea.str_descripcion.Contains(temp));
                        }
                        //predicate = predicate.And(predicateDescription.Expand());
                    }

                    if (!String.IsNullOrEmpty(ideaCriteria.str_titulo) || !String.IsNullOrEmpty(ideaCriteria.str_descripcion))
                        predicate = predicate.And(predicateTitle.Expand());

                }

                //Ejecuta la consulta de selección de organizaciones
                var getIdeas = from Ideas in ideasBD.vw_Ideas.AsExpandable().Where(predicate)
                               select Ideas;

                List<vw_Ideas> lstIdeas = new List<vw_Ideas>();

                if (getIdeas != null && getIdeas.Count() > 0)
                    lstIdeas = getIdeas.ToList<vw_Ideas>();

                //Devuelve el listado de organizaciones
                return lstIdeas;

            }
            catch (Exception ex)
            {
                throw (ex);
            }

        }

        /// <summary>
        /// Obtiene el detalle de una idea a partir de su clave primaria
        /// </summary>
        /// <param name="idIdea"></param>
        /// <returns></returns>
        internal tbl_ideas getIdeaDetail(string idIdea)
        {
            try
            {
                var ideaDetail = from ideaDet in ideasBD.tbl_ideas
                                 where ideaDet.pk_uid_id == new Guid(idIdea)
                                 select ideaDet;

                if (ideaDetail.Count() > 0)
                    return ideaDetail.First<tbl_ideas>();
                else
                    return null;

            }
            catch (Exception ex)
            {
                throw (ex);
            }
        }

        /// <summary>
        /// Actualiza una idea sobre la base de datos.
        /// </summary>
        /// <param name="ideaDetail"></param>
        internal void updateIdea(tbl_ideas ideaDetail)
        {
            try
            {
            //http://blogs.msdn.com/b/wriju/archive/2008/10/16/ado-net-entity-insert-update-and-delete-with-relationship.aspx

                tbl_ideas ideaDet = (from idea in ideasBD.tbl_ideas
                                             where idea.pk_uid_id == ideaDetail.pk_uid_id
                                             select idea).Single();

                if (ideaDet != null)
                {
                    ideaDet.bol_archivada = ideaDetail.bol_archivada;
                    ideaDet.dat_fechaCreacion = ideaDetail.dat_fechaCreacion;
                    ideaDet.dat_fechalimite = ideaDetail.dat_fechalimite;
                    ideaDet.epk_lng_idEstado = ideaDetail.epk_lng_idEstado;
                    ideaDet.epk_lng_razonArchivo = ideaDetail.epk_lng_razonArchivo;
                    ideaDet.epk_uid_propietario = ideaDetail.epk_uid_propietario;
                    ideaDet.str_descripcion = ideaDetail.str_descripcion;
                    ideaDet.str_titulo = ideaDetail.str_titulo;
                    List<tbl_rel_ideatema> lstideaThemes = ideaDet.tbl_rel_ideatema.ToList<tbl_rel_ideatema>();
                    //Elimina la relación de ideas - temas anterior 
                    foreach (tbl_rel_ideatema deleteIdeaTheme in lstideaThemes)
                    {
                        ideaDet.tbl_rel_ideatema.Remove(deleteIdeaTheme);
                        ideasBD.tbl_rel_ideatema.DeleteObject(deleteIdeaTheme);
                    }
                    //Relaciona los nuevos temas con las idea en cursos
                    for (int i = 0; i < ideaDetail.tbl_rel_ideatema.Count; i++)
                    {
                        tbl_rel_ideatema ideaTema = new tbl_rel_ideatema();
                        ideaTema.pk_uid_id = Guid.NewGuid();
                        ideaTema.epk_uid_idea = ideaDetail.tbl_rel_ideatema.ElementAt(i).epk_uid_idea;
                        ideaTema.epk_uid_tema = ideaDetail.tbl_rel_ideatema.ElementAt(i).epk_uid_tema;
                        
                        ideaDet.tbl_rel_ideatema.Add(ideaTema);
                    }
                    //Elimina la relación de usuarios - ideas existente
                    List<tbl_rel_ideausuario> lstIdeaUsers = ideaDet.tbl_rel_ideausuario.ToList<tbl_rel_ideausuario>();
                    foreach (tbl_rel_ideausuario deleteIdeaUser in lstIdeaUsers)
                    {
                        ideaDet.tbl_rel_ideausuario.Remove(deleteIdeaUser);
                        ideasBD.tbl_rel_ideausuario.DeleteObject(deleteIdeaUser);
                    }
                    //Añade la nueva relación de usuarios e ideas
                    for (int i = 0; i < ideaDetail.tbl_rel_ideausuario.Count; i++)
                    {
                        tbl_rel_ideausuario ideaUser = new tbl_rel_ideausuario();
                        ideaUser.pk_uid_id = Guid.NewGuid();
                        ideaUser.epk_uid_idea = ideaDetail.tbl_rel_ideausuario.ElementAt(i).epk_uid_idea;
                        ideaUser.epk_uid_usuario = ideaDetail.tbl_rel_ideausuario.ElementAt(i).epk_uid_usuario;

                        ideaDet.tbl_rel_ideausuario.Add(ideaUser);
                    }

                    ideasBD.SaveChanges(SaveOptions.AcceptAllChangesAfterSave);
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }

        }

        /// <summary>
        /// Archiva una idea
        /// </summary>
        /// <param name="uid_idea"></param>
        /// <param name="id_reason"></param>
        /// <param name="id_estado"></param>
        internal void updateIdeaReasonArchive(Guid uid_idea, int id_reason, int id_estado)
        {
            try
            {
                tbl_ideas ideaDet = (from idea in ideasBD.tbl_ideas
                                     where idea.pk_uid_id == uid_idea
                                     select idea).Single();

                if (ideaDet != null)
                {
                    ideaDet.bol_archivada = true;
                    ideaDet.epk_lng_razonArchivo = id_reason;
                    ideaDet.epk_lng_idEstado = id_estado;
                }

                ideasBD.SaveChanges(SaveOptions.AcceptAllChangesAfterSave);

            }
            catch (Exception ex)
            {
                throw (ex);
            }
        }

        /// <summary>
        /// Obtiene un listado de ideas ordenadas según su puntuación
        /// </summary>
        /// <param name="idTema"></param>
        /// <param name="int_archivo"></param>
        /// <returns></returns>
        internal List<vw_rankingIdeas> getRankingIdeas(string  idTema, int int_archivo)
        {
            try
            {
                var predicate = PredicateBuilder.True<tbl_rel_ideatema>();
                var predicateRanking  = PredicateBuilder.True<vw_rankingIdeas>();

                //Si se ha introducido algún criterio de busqueda. Añade criterios al predicado
                if (!String.IsNullOrWhiteSpace(idTema))
                    predicate = predicate.And(tema => tema.epk_uid_tema == new Guid(idTema));
                    
                if (int_archivo == 0)
                        predicateRanking = predicateRanking.And(idea => idea.bol_archivada == false);
                if (int_archivo == 1)
                    predicateRanking = predicateRanking.And(idea => idea.bol_archivada == true);


                //Ejecuta la consulta de selección de organizaciones
                var getRankingIdeas = from Ideas in ideasBD.vw_rankingIdeas.AsExpandable().Where(predicateRanking)
                                      from Temas in ideasBD.tbl_rel_ideatema.AsExpandable().Where(predicate)
                                      where Ideas.pk_uid_id == Temas.epk_uid_idea
                                      orderby Ideas.ValorMedioIdea descending
                                      select Ideas;


                List<vw_rankingIdeas> lstIdeas = new List<vw_rankingIdeas>();

                if (getRankingIdeas != null && getRankingIdeas.Count() > 0)
                {
                    lstIdeas = getRankingIdeas.Distinct().ToList<vw_rankingIdeas>();
                    lstIdeas = lstIdeas.OrderByDescending(p => p.ValorMedioIdea).ToList();
                }

                //Devuelve el listado de organizaciones
                return lstIdeas;

            }
            catch (Exception ex)
            {
                throw (ex);
            }

        }
    }
}