﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using BraStor_WebRole.Model;
using System.Configuration;
using LinqKit;
using System.Data.Objects;

namespace BraStor_WebRole.Dal
{
    
    public class DalOrganizaciones
    {
        UOC_IDEASEntities ideasBD;

        public DalOrganizaciones()
        {
            string str_connection = ConfigurationManager.ConnectionStrings["UOC_IDEASEntities"].ConnectionString;
            ideasBD = new UOC_IDEASEntities(str_connection);
        }

        /// <summary>
        /// Trae un listado de todas las organizaciones en función de los criterios establecidos en organizationCriteria
        /// </summary>
        /// <param name="organizationCriteria"></param>
        /// <returns></returns>
        internal List<tbl_organizaciones> getListOrganizations(tbl_organizaciones organizationCriteria)
        {
            try
            {
                var predicate = PredicateBuilder.True<tbl_organizaciones>();
                
                //Si se ha introducido algún criterio de busqueda. Añade criterios al predicado
                if (organizationCriteria != null)
                {
                    if (!String.IsNullOrWhiteSpace(organizationCriteria.str_nombre))
                        predicate = predicate.And(org => org.str_nombre.Contains(organizationCriteria.str_nombre));

                    if (!String.IsNullOrWhiteSpace(organizationCriteria.str_provincia))
                        predicate = predicate.And(org => org.str_provincia.Contains(organizationCriteria.str_provincia));

                    if (!String.IsNullOrWhiteSpace(organizationCriteria.str_pais))
                        predicate = predicate.And(org => org.str_pais.Contains(organizationCriteria.str_pais));
                }

                //Ejecuta la consulta de selección de organizaciones
                var getOrganizations = from organizations in ideasBD.tbl_organizaciones.AsExpandable().Where(predicate)
                                       select organizations;

                List<tbl_organizaciones> lstOrganizations = new List<tbl_organizaciones>();

                if (getOrganizations != null && getOrganizations.Count() > 0)
                    lstOrganizations = getOrganizations.ToList<tbl_organizaciones>();

                //Devuelve el listado de organizaciones
                return lstOrganizations;

            }
            catch (Exception ex)
            {
                throw (ex);
            }
        }

        /// <summary>
        /// Obtiene el detalle de una organización a partir de su tabla primaria
        /// </summary>
        /// <param name="uid_id"></param>
        /// <returns></returns>
        internal tbl_organizaciones getDetailOrganization(string uid_id)
        {
            try
            {
                var organizationDet = from orgDet in ideasBD.tbl_organizaciones
                                      where orgDet.pk_uid_id == new Guid(uid_id)
                                      select orgDet;

                if (organizationDet.Count() > 0)
                {
                    return organizationDet.First<tbl_organizaciones>();
                }
                else
                    return null;

            }
            catch (Exception ex)
            {
                throw ex;
            }
        }

        /// <summary>
        /// Actualiza los datos de la tabla organización
        /// </summary>
        /// <param name="organizationDetail"></param>
        internal void updateOrganization(tbl_organizaciones organizationDetail)
        {
            try{

                tbl_organizaciones orgDet = (from org in ideasBD.tbl_organizaciones
                                where org.pk_uid_id == organizationDetail.pk_uid_id
                                select org).Single();

                if (orgDet != null)
                {
                    orgDet.str_codigo = organizationDetail.str_codigo;
                    orgDet.str_nombre = organizationDetail.str_nombre;
                    orgDet.str_telefono = organizationDetail.str_telefono;
                    orgDet.str_fax = organizationDetail.str_fax;
                    orgDet.str_cif = organizationDetail.str_cif;
                    orgDet.str_direccion = organizationDetail.str_direccion;
                    orgDet.str_cp = organizationDetail.str_cp;
                    orgDet.str_poblacion = organizationDetail.str_poblacion;
                    orgDet.str_provincia = organizationDetail.str_provincia;
                    orgDet.str_pais = organizationDetail.str_pais;

                    ideasBD.SaveChanges(SaveOptions.AcceptAllChangesAfterSave);
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
        }

        /// <summary>
        /// Inserta una nueva organización dentro de la base de datos
        /// </summary>
        /// <param name="newOrganization"></param>
        internal void newOrganization(tbl_organizaciones newOrganization)
        {
            try
            {
                newOrganization.pk_uid_id = Guid.NewGuid();
                ideasBD.AddTotbl_organizaciones(newOrganization);

                ideasBD.SaveChanges(SaveOptions.AcceptAllChangesAfterSave);
            }
            catch (Exception ex)
            {
                throw (ex);
            }
        }
    }
}