﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using BraStor_WebRole.Model;
using System.Configuration;
using System.Data.Objects;

namespace BraStor_WebRole.Dal
{
    public class DalThemes
    {
        UOC_IDEASEntities ideasBD;

        public DalThemes()
        {
            string str_connection = ConfigurationManager.ConnectionStrings["UOC_IDEASEntities"].ConnectionString;
            ideasBD = new UOC_IDEASEntities(str_connection);
        }

        /// <summary>
        /// Inserta un nuevo tema dentro de la base de datos
        /// </summary>
        /// <param name="newTheme"></param>
        internal void newThemes(Model.tbl_temas newTheme)
        {
            try
            {
                newTheme.pk_uid_id = Guid.NewGuid();
                ideasBD.AddTotbl_temas(newTheme);

                ideasBD.SaveChanges(SaveOptions.AcceptAllChangesAfterSave);
            }
            catch (Exception ex)
            {
                throw (ex);
            }

        }

        /// <summary>
        /// Obtiene un listado de todos los temas
        /// </summary>
        /// <returns></returns>
        internal List<tbl_temas> getAllThemes()
        {
            try
            {
                //Ejecuta la consulta de selección de temas
                var getThemes = from themes in ideasBD.tbl_temas
                                       select themes;

                List<tbl_temas> lstThemes = new List<tbl_temas>();

                if (getThemes != null && getThemes.Count() > 0)
                    lstThemes = getThemes.ToList<tbl_temas>();

                //Devuelve el listado de organizaciones
                return lstThemes;

            }
            catch (Exception ex)
            {
                throw (ex);
            }
        }

        /// <summary>
        /// Obtiene un listado de todos los temas que se encuentran activos
        /// </summary>
        /// <returns></returns>
        internal List<tbl_temas> getActiveThemes()
        {
            try
            {
                //Ejecuta la consulta de selección de temas
                var getThemes = from themes in ideasBD.tbl_temas
                                where themes.bol_activo == true
                                select themes;

                List<tbl_temas> lstThemes = new List<tbl_temas>();

                if (getThemes != null && getThemes.Count() > 0)
                    lstThemes = getThemes.ToList<tbl_temas>();

                //Devuelve el listado de organizaciones
                return lstThemes;

            }
            catch (Exception ex)
            {
                throw (ex);
            }
        }


        /// <summary>
        /// Actualiza los datos de un tema
        /// </summary>
        /// <param name="themeDetail"></param>
        internal void updateThemes(tbl_temas themeDetail)
        {
            try
            {

                tbl_temas theDet = (from theme in ideasBD.tbl_temas
                                    where theme.pk_uid_id == themeDetail.pk_uid_id
                                    select theme).Single();

                if (theDet != null)
                {
                    theDet.str_codigo = themeDetail.str_codigo;
                    theDet.str_nombre = themeDetail.str_nombre;
                    theDet.bol_activo = themeDetail.bol_activo;

                    ideasBD.SaveChanges(SaveOptions.AcceptAllChangesAfterSave);
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
        }

        /// <summary>
        /// Obtiene toda la información de un tema
        /// </summary>
        /// <param name="themeUid"></param>
        /// <returns></returns>
        internal tbl_temas getDetailTheme(string themeUid)
        {

            try
            {
                var themeDet = from theme in ideasBD.tbl_temas
                               where theme.pk_uid_id == new Guid(themeUid)
                               select theme;

                if (themeDet.Count() > 0)
                {
                    return themeDet.First<tbl_temas>();
                }
                else
                    return null;

            }
            catch (Exception ex)
            {
                throw ex;
            }
        }
    }
}