﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using BraStor_WebRole.Model;
using System.Configuration;
using LinqKit;
using System.Web.Security;
using System.Data.Objects;

namespace BraStor_WebRole.Dal
{
    public class DalUsers
    {
        UOC_IDEASEntities ideasBD;

        public DalUsers()
        {
            string str_connection = ConfigurationManager.ConnectionStrings["UOC_IDEASEntities"].ConnectionString;
            ideasBD = new UOC_IDEASEntities(str_connection);
        }

        /// <summary>
        /// Inserta una nueva personalización de usuario dentro de la base de datos
        /// </summary>
        /// <param name="persUserNew"></param>
        internal void newPersonalization(tbl_personalizacionUsuarios persUserNew)
        {
            try
            {
                ideasBD.AddTotbl_personalizacionUsuarios(persUserNew);

                ideasBD.SaveChanges(SaveOptions.AcceptAllChangesAfterSave);
            }
            catch (Exception ex)
            {
                throw (ex);
            }
        }

        /// <summary>
        /// Trae un listado de todos los usuarios que se enecuentran dados de alta en el sistema en función de UserCriteria
        /// Inicialmente trae un listado de todos los usuarios, no se ha establecido un criterio de búsqueda
        /// </summary>
        /// <param name="organizationCriteria"></param>
        /// <returns></returns>
        internal List<vw_Users> getListUsers(vw_Users userCriteria)
        {
            try
            {
                var predicate = PredicateBuilder.True<vw_Users>();

                if (!String.IsNullOrWhiteSpace(userCriteria.UserName))
                    predicate = predicate.And(usr => usr.UserName == userCriteria.UserName);

                //Ejecuta la consulta de selección de usuarios
                var getUsers = from users in ideasBD.vw_Users.AsExpandable().Where(predicate)
                                       select users;

                List<vw_Users> lstUsers = new List<vw_Users>();

                if (getUsers != null && getUsers.Count() > 0)
                    lstUsers = getUsers.ToList<vw_Users>();

                //Devuelve el listado de usuarios
                return lstUsers;
            }
            catch (Exception ex)
            {
                throw (ex);
            }
        }

        /// <summary>
        /// Recupera una lista de todos los usuarios que están activos en el sistema
        /// </summary>
        /// <returns></returns>
        internal List<vw_Users> getListActiveUsers()
        {
            try
            {
                var predicate = PredicateBuilder.True<vw_Users>();

                predicate = predicate.And(usr => usr.IsLockedOut == false);

                //Ejecuta la consulta de selección de usuarios
                var getUsers = from users in ideasBD.vw_Users.AsExpandable().Where(predicate)
                               select users;

                List<vw_Users> lstUsers = new List<vw_Users>();

                if (getUsers != null && getUsers.Count() > 0)
                    lstUsers = getUsers.ToList<vw_Users>();

                //Devuelve el listado de usuarios
                return lstUsers;
            }
            catch (Exception ex)
            {
                throw (ex);
            }
        }

        /// <summary>
        /// Obtiene el detalle de un usuario a partir de su clave primaria
        /// </summary>
        /// <param name="uid_id"></param>
        /// <returns></returns>
        internal vw_Users getDetailUser(string uid_id)
        {
            try
            {
                var userDetail = from userDet in ideasBD.vw_Users
                                 where userDet.UserId == new Guid(uid_id)
                                      select userDet;

                if (userDetail.Count() > 0)
                {
                    return userDetail.First<vw_Users>();
                }
                else
                    return null;

            }
            catch (Exception ex)
            {
                throw ex;
            }
        }


        /// <summary>
        /// Obtiene el detalle de un usuario a partir de su nombre
        /// </summary>
        /// <param name="uid_id"></param>
        /// <returns></returns>
        internal vw_Users getDetailUserByName(string userName)
        {
            try
            {
                var userDetail = from userDet in ideasBD.vw_Users
                                 where userDet.UserName == userName
                                 select userDet;

                if (userDetail.Count() > 0)
                {
                    return userDetail.First<vw_Users>();
                }
                else
                    return null;

            }
            catch (Exception ex)
            {
                throw ex;
            }
        }



        /// <summary>
        /// Actualiza los datos de un usuario
        /// </summary>
        /// <param name="userDetail"></param>
        internal void updateUser(vw_Users userDetail)
        {
            try
            {
                //Actualiza los datos del usuario
                MembershipUser mbsUser =  Membership.GetUser(userDetail.UserId);

                mbsUser.Email = userDetail.Email;
                Membership.UpdateUser(mbsUser);

                //Actualiza el rol en caso de que corresponda
                if (!Roles.IsUserInRole(mbsUser.UserName,userDetail.RoleName))
                {
                    //Borra los anteriores roles creados
                    string[] str_roles = Roles.GetRolesForUser(mbsUser.UserName);
                    Roles.RemoveUserFromRoles(mbsUser.UserName, str_roles);

                    //Añade el nuevo Role creado
                    Roles.AddUserToRole(mbsUser.UserName, userDetail.RoleName);
                }

                var existsUserPer = from userPers in ideasBD.tbl_personalizacionUsuarios
                                    where userPers.pk_uid_id == userDetail.UserId
                                    select userPers;

                if (existsUserPer!= null && existsUserPer.Count() > 0)
                    this.updateDetail(userDetail);
                else
                {
                    //Si no ha encontrado personalización para este usuario la inserta
                    tbl_personalizacionUsuarios personalizationDetail = new tbl_personalizacionUsuarios();
                    personalizationDetail.pk_uid_id = userDetail.UserId;
                    personalizationDetail.str_nombre = userDetail.str_nombre;
                    personalizationDetail.str_apellidos = userDetail.str_apellidos;
                    personalizationDetail.str_telefono = userDetail.str_telefono;
                    personalizationDetail.uid_organizacion = userDetail.uid_organizacion;

                    newPersonalization(personalizationDetail);
                }
            }
            catch (Exception ex)
            {
                throw (ex);
            }

        }

        /// <summary>
        /// Actualiza los datos de la tabla de personalización de usuarios
        /// </summary>
        /// <param name="userDetail"></param>
        internal void updateDetail(vw_Users userDetail)
        {
            try
            {

                tbl_personalizacionUsuarios userPerDetail = (from userPers in ideasBD.tbl_personalizacionUsuarios
                                                             where userPers.pk_uid_id == userDetail.UserId
                                                             select userPers).Single();

                if (userPerDetail != null)
                {
                    userPerDetail.str_nombre = userDetail.str_nombre;
                    userPerDetail.str_apellidos = userDetail.str_apellidos;
                    userPerDetail.str_telefono = userDetail.str_telefono;
                    userPerDetail.uid_organizacion = userDetail.uid_organizacion;

                    ideasBD.SaveChanges(SaveOptions.AcceptAllChangesAfterSave);
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
        }

 
    }
}