﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using BraStor_WebRole.Model;
using BraStor_WebRole.Bll;
using System.Drawing;


namespace BraStor_WebRole.Ideas
{
    public partial class IdeasDetail : System.Web.UI.Page
    {
        public static tbl_ideas IdeaDetail = new tbl_ideas();
        public static BllIdeas bIdeas = new BllIdeas();
        public static List<vw_Comentarios> lstComments = new List<vw_Comentarios>();
        public static string idIdea;

        protected void Page_Load(object sender, EventArgs e)
        {
            //Cargar los datos de la idea
            if (!IsPostBack)
            {
                idIdea = Request.QueryString["ID"];
                if (!String.IsNullOrEmpty(idIdea))
                {
                    //Hace invisible la etiqueta de error
                    lbl_error.Visible = false;
                    lbl_error.ForeColor = Color.Blue;
                    lbl_error.Text = "";

                    //Recoge los datos de la idea
                    IdeaDetail = bIdeas.getIdeaDetail(idIdea);

                    showIdea(IdeaDetail);

                    //Obtener todos los comentarios relacionados con la idea
                    lstComments = bIdeas.getComentariosIdea(idIdea);

                    this.gv_comentarios.DataSource = lstComments;
                    this.gv_comentarios.DataBind();

                    //Valoración media
                    Double dbl_meanValue = bIdeas.getMeanValueIdea(idIdea);
                    if (dbl_meanValue < 5)
                        this.txt_average.ForeColor = Color.Red;
                    else
                        this.txt_average.ForeColor = Color.Blue;
                    this.txt_average.Text = dbl_meanValue.ToString();

                    //Valoración del usuario
                    double dbl_noteUser = bIdeas.getNoteOfIdea_User(idIdea, Page.User.Identity.Name);
                    if (dbl_noteUser < 5)
                        this.txt_myPoint.ForeColor = Color.Red;
                    else
                        this.txt_myPoint.ForeColor = Color.Blue;
                    this.txt_myPoint.Text = dbl_noteUser.ToString();

                }

                //Obtiene los datos del usuario
                vw_Users userData =  bIdeas.getUserDetailByName(Page.User.Identity.Name);
                if (userData != null)
                {
                    //Se permite archivar y desarchivar si el usuario es el propietario de la idea
                    // o tiene Role de Moderador

                    if (User.IsInRole("Moderador") || userData.UserId == IdeaDetail.epk_uid_propietario)
                    {
                        if (IdeaDetail.bol_archivada != null && Convert.ToBoolean(IdeaDetail.bol_archivada))
                            this.lnk_archivar.Visible = false;
                        else
                            this.lnk_archivar.Visible = true;
                    }
                    else
                        this.lnk_archivar.Visible = false;

                }
                else
                    this.lnk_archivar.Visible = false;

                //Solo puede valorar la idea, aquellos que han sido asignados a ella.
                //La fecha limite no se puede sobrepasar y no debe estar archivad

                vw_Users userDet = bIdeas.getUserDetailByName(Page.User.Identity.Name);
                tbl_rel_ideausuario relIdUsr = null;
                try
                {
                    relIdUsr = IdeaDetail.tbl_rel_ideausuario.Where(x => x.epk_uid_usuario == userDet.UserId).Single();
                }
                catch (Exception)
                {
                    relIdUsr = null;
                }

                if (relIdUsr == null)
                {
                    this.blnk_note.Visible = false;
                    this.blnk_addComment.Visible = false;
                }
                else
                {
                    Boolean bArchivada = IdeaDetail.bol_archivada == null ? false : Convert.ToBoolean(IdeaDetail.bol_archivada);
                    if (DateTime.Today <= IdeaDetail.dat_fechalimite && !bArchivada)
                    {
                        this.blnk_note.Visible = true;
                        this.blnk_addComment.Visible = true;
                    }
                    else
                    {
                        this.blnk_note.Visible = false;
                        this.blnk_addComment.Visible = false;
                    }
                }
            }
            this.lbl_error.Visible = false;

        }

        /// <summary>
        /// Coloca los datos de la idea en los controles correspondientes
        /// </summary>
        /// <param name="IdeaDetail"></param>
        private void showIdea(tbl_ideas ideaDetail)
        {
            try
            {
                this.lbl_error.Visible = false;

                this.txt_title.Text = ideaDetail.str_titulo;
                this.txt_creationDate.Text = ideaDetail.dat_fechaCreacion.ToString("dd/MM/yyyy");
                this.txt_limitDate.Text = ideaDetail.dat_fechalimite.ToString();
                this.txt_description.Text = ideaDetail.str_descripcion;
                this.txt_status.Text = ideaDetail.tbl_sys_estadoIdea.str_nombre;
                this.chk_Archivada.Checked = ideaDetail.bol_archivada == null ? false : Convert.ToBoolean(ideaDetail.bol_archivada);
                //TEMAS
                foreach (tbl_rel_ideatema ideaTema in ideaDetail.tbl_rel_ideatema)
                {
                    tbl_temas tema = ideaTema.tbl_temasReference.Value;

                    ListItem item = new ListItem(tema.str_nombre, tema.pk_uid_id.ToString());
                    lstThemes.Items.Add(item);

                }

                
            }
            catch (Exception ex)
            {
                lbl_error.Visible = true;
                lbl_error.ForeColor = Color.Red;
                lbl_error.Text = ex.Message;
            }
        }

        protected void InsertCancelButton_Click(object sender, EventArgs e)
        {
            Response.Redirect("~/Ideas/IdeasIndex.aspx");
        }


 
        protected void blnk_okArchivo_Click(object sender, EventArgs e)
        {
            this.lbl_error.Visible = false;

            bIdeas.archivarIdea(IdeaDetail.pk_uid_id, Convert.ToInt32(this.ddl_razonArchivo.SelectedValue));
            
            //Recoge los datos de la idea
            IdeaDetail = bIdeas.getIdeaDetail(idIdea);
            this.chk_Archivada.Checked = Convert.ToBoolean(IdeaDetail.bol_archivada);
            this.txt_status.Text = IdeaDetail.tbl_sys_estadoIdea.str_nombre;
            if (this.chk_Archivada.Checked)
                this.lnk_archivar.Visible = false;

            this.updPan_archivo.Update();
        }

        /// <summary>
        /// Valoración de una idea dada por el usuario
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void lnk_okValorar_Click(object sender, EventArgs e)
        {
            this.lbl_error.Visible = false;

            Int32 int_nota = 0;

            //Recoge la valoración del usuario
            if (this.rb_val1.Checked)
                int_nota = 0;
            if (this.rb_val2.Checked)
                int_nota = 1;
            if (this.rb_val3.Checked)
                int_nota = 2;
            if (this.rb_val4.Checked)
                int_nota = 3;
            if (this.rb_val5.Checked)
                int_nota = 4;
            if (this.rb_val6.Checked)
                int_nota = 5;
            if (this.rb_val7.Checked)
                int_nota = 6;
            if (this.rb_val8.Checked)
                int_nota = 7;
            if (this.rb_val9.Checked)
                int_nota = 8;
            if (this.rb_val10.Checked)
                int_nota = 9;
            if (this.rb_val11.Checked)
                int_nota = 10;

       

            bIdeas.setValueIdea(IdeaDetail.pk_uid_id, Page.User.Identity.Name, int_nota);

            this.txt_myPoint.Text = bIdeas.getNoteOfIdea_User(IdeaDetail.pk_uid_id.ToString(),Page.User.Identity.Name).ToString();
            this.txt_average.Text = bIdeas.getMeanValueIdea(IdeaDetail.pk_uid_id.ToString()).ToString();
            this.updPan_Valor.Update();
        }

        protected void blnk_addComment_Click(object sender, EventArgs e)
        {
            this.lbl_error.Visible = false;
            this.txt_dateComment.Text = DateTime.Today.ToShortDateString();
            updComments.Update();
            comment_ModalPopupExtender.Show();
        }

        protected void blnk_okComment_Click(object sender, EventArgs e)
        {
            //Recoge los datos del comentario
            string str_comments = this.txt_comment.Text;
            if (String.IsNullOrEmpty(str_comments))
            {
                this.lbl_error.Text = "Por favor, Introduzca un comentario";
                this.lbl_error.ForeColor = Color.Red;
                this.lbl_error.Visible = true;
                return;
            }

            bIdeas.addComments(IdeaDetail.pk_uid_id, Page.User.Identity.Name, str_comments);

            List<vw_Comentarios> lst_comments = bIdeas.getComentariosIdea(IdeaDetail.pk_uid_id.ToString());
            this.gv_comentarios.DataSource = lst_comments;
            this.gv_comentarios.DataBind();

            updPan_comments.Update();
        }

        

    }
}