﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using BraStor_WebRole.Bll;
using BraStor_WebRole.Model;
using System.Drawing;

namespace BraStor_WebRole.Ideas
{
    public partial class IdeasEdit : System.Web.UI.Page
    {
        public static tbl_ideas IdeaDetail = new tbl_ideas();
        public static BllIdeas bIdeas = new BllIdeas();

        protected void Page_Load(object sender, EventArgs e)
        {
            //Cargar los datos de la idea
            if (!IsPostBack)
            {
                string idIdea = Request.QueryString["ID"];
                if (!String.IsNullOrEmpty(idIdea))
                {
                    //Hace invisible la etiqueta de error
                    lbl_error.Visible = false;
                    lbl_error.ForeColor = Color.Blue;
                    lbl_error.Text = "";

                    //Recoge los datos de la idea
                    IdeaDetail = bIdeas.getIdeaDetail(idIdea);

                    showIdea(IdeaDetail);

                    //Enviar eMail, por defecto true
                    this.chk_mail.Checked = true;
                }
            }

        }

        /// <summary>
        /// Coloca los datos de la idea en los controles correspondientes
        /// </summary>
        /// <param name="IdeaDetail"></param>
        private void showIdea(tbl_ideas ideaDetail)
        {
            try
            {
                this.txt_title.Text = ideaDetail.str_titulo;
                this.txt_creationDate.Text = ideaDetail.dat_fechaCreacion.ToString("dd/MM/yyyy");
                this.txt_limitDate.Text = ideaDetail.dat_fechalimite.ToString();
                this.txt_description.Text = ideaDetail.str_descripcion;
                this.ddl_status.SelectedValue = ideaDetail.epk_lng_idEstado.ToString();
                this.chk_Archivada.Checked = ideaDetail.bol_archivada==null?false:Convert.ToBoolean(ideaDetail.bol_archivada);
                foreach (tbl_rel_ideausuario ideaUser in ideaDetail.tbl_rel_ideausuario)
                {
                   vw_Users userDetail =  bIdeas.getUserDetail(ideaUser.epk_uid_usuario.ToString());

                    ListItem item = new ListItem(userDetail.UserName,userDetail.UserId.ToString());
                    lst_users.Items.Add(item);
                }
                
                foreach (tbl_rel_ideatema ideaTema in ideaDetail.tbl_rel_ideatema)
                {
                    tbl_temas tema = ideaTema.tbl_temasReference.Value;

                    ListItem item = new ListItem(tema.str_nombre, tema.pk_uid_id.ToString());
                    lstThemes.Items.Add(item);

                }
            }
            catch (Exception ex)
            {
                lbl_error.Visible = true;
                lbl_error.ForeColor = Color.Red;
                lbl_error.Text = ex.Message;
            }
        }

        /// <summary>
        /// Procede a realizar los cambios necesarios sobre la Idea
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void UpdateButton_Click(object sender, EventArgs e)
        {
            try
            {
                BllIdeas bIdeas = new BllIdeas();

                IdeaDetail.str_titulo = this.txt_title.Text;
                IdeaDetail.dat_fechalimite = Convert.ToDateTime(this.txt_limitDate.Text);
                IdeaDetail.str_descripcion = this.txt_description.Text;

                IdeaDetail.epk_lng_idEstado = Convert.ToInt32(this.ddl_status.SelectedValue);

                //Temas asociado
                IdeaDetail.tbl_rel_ideatema.Clear();
                for (int it = 0; it < lstThemes.Items.Count; it++)
                {
                    tbl_rel_ideatema uidTheme = new tbl_rel_ideatema();
                    uidTheme.epk_uid_tema = new Guid(lstThemes.Items[it].Value);

                    IdeaDetail.tbl_rel_ideatema.Add(uidTheme);
                }

                //Usuarios asociados
                IdeaDetail.tbl_rel_ideausuario.Clear();
                for (int it = 0; it < lst_users.Items.Count; it++)
                {
                    tbl_rel_ideausuario uidUser = new tbl_rel_ideausuario();
                    uidUser.epk_uid_usuario = new Guid(lst_users.Items[it].Value);

                    IdeaDetail.tbl_rel_ideausuario.Add(uidUser);
                }

                Boolean bol_email = this.chk_mail.Checked;
                bIdeas.updateIdea(IdeaDetail,bol_email);

                Response.Redirect("~/Ideas/IdeasIndex.aspx");
            }
            catch (Exception ex)
            {
                this.lbl_error.ForeColor = Color.Red;
                this.lbl_error.Text = "Se ha producido un error durante la inserción de los datos. [" + ex.Message + "]";
                this.lbl_error.Visible = true;
            }

        }

        /// <summary>
        /// Añade a la lista los temas que se están relacionando con la idea
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void btn_addTheme_Click(object sender, EventArgs e)
        {
            this.lstThemes.ClearSelection();
            ListItem selectedTheme = this.ddl_themes.Items[this.ddl_themes.SelectedIndex];

            //Solo añade el tema cuando no se encuentra en la lista
            int position = this.lstThemes.Items.IndexOf(selectedTheme);

            if (position < 0)
                this.lstThemes.Items.Add(selectedTheme);

        }

        /// <summary>
        /// Elimina de la lista el tema seleccionado
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void btn_deleteTheme_Click(object sender, EventArgs e)
        {
            int selectedIndex = this.lstThemes.SelectedIndex;
            if (selectedIndex >= 0)
                this.lstThemes.Items.RemoveAt(selectedIndex);
        }

        /// <summary>
        /// Añade un usuario a lista de usuarios
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void btn_addUser_Click(object sender, EventArgs e)
        {
            this.lst_users.ClearSelection();
            ListItem selectedUser = this.ddl_users.Items[this.ddl_users.SelectedIndex];

            //Solo añade el tema cuando no se encuentra en la lista
            int position = this.lst_users.Items.IndexOf(selectedUser);

            if (position < 0)
                this.lst_users.Items.Add(selectedUser);
        }

        /// <summary>
        /// Elimina de la lista al usuario seleccinado
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void btn_deleteUser_Click(object sender, EventArgs e)
        {
            int selectedIndex = this.lst_users.SelectedIndex;
            if (selectedIndex >= 0)
                this.lst_users.Items.RemoveAt(selectedIndex);
        }

        protected void InsertCancelButton_Click(object sender, EventArgs e)
        {
            Response.Redirect("~/Ideas/IdeasIndex.aspx");
        }
    }
}