﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using BraStor_WebRole.Model;
using BraStor_WebRole.Bll;
using System.Drawing;

namespace BraStor_WebRole.Ideas
{
    public partial class IdeasNew : System.Web.UI.Page
    {
        protected void Page_Load(object sender, EventArgs e)
        {
            if (!IsPostBack)
            {
                this.lbl_error.Visible = false;
            }
        }

        /// <summary>
        /// Recoge los datos introducidos en la pantalla para poder iniciar el proceso de dar de alta una nueva idea
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void InsertButton_Click(object sender, EventArgs e)
        {
            try
            {
                BllIdeas bIdeas = new BllIdeas();

                tbl_ideas newIdea = new tbl_ideas();

                newIdea.str_titulo = this.txt_title.Text;
                newIdea.dat_fechalimite = Convert.ToDateTime(this.txt_limitDate.Text);
                newIdea.str_descripcion = this.txt_description.Text;

                //Temas asociado
                for (int it = 0;it<lstThemes.Items.Count;it++)
                {
                    tbl_rel_ideatema uidTheme = new tbl_rel_ideatema();
                    uidTheme.epk_uid_tema = new Guid(lstThemes.Items[it].Value);

                    newIdea.tbl_rel_ideatema.Add(uidTheme);
                }

                //Usuarios asociados
                for (int it = 0; it < lst_users.Items.Count; it++)
                {
                    tbl_rel_ideausuario uidUser = new tbl_rel_ideausuario();
                    uidUser.epk_uid_usuario = new Guid(lst_users.Items[it].Value);

                    newIdea.tbl_rel_ideausuario.Add(uidUser);
                }

                bIdeas.createIdea(newIdea);
                
                Response.Redirect("~/Ideas/IdeasIndex.aspx");
            }
            catch (Exception ex)
            {
                this.lbl_error.ForeColor = Color.Red;
                this.lbl_error.Text = "Se ha producido un error durante la inserción de los datos. [" + ex.Message + "]";
                this.lbl_error.Visible = true;
            }

        }

        protected void InsertCancelButton_Click(object sender, EventArgs e)
        {
            Response.Redirect("~/Ideas/IdeasIndex.aspx");
        }

        /// <summary>
        /// Añade a la lista los temas que se están relacionados con la idea
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void btn_addTheme_Click(object sender, EventArgs e)
        {
            this.lstThemes.ClearSelection();
            ListItem selectedTheme = this.ddl_themes.Items[this.ddl_themes.SelectedIndex];

            //Solo añade el tema cuando no se encuentra en la lista
            int position = this.lstThemes.Items.IndexOf(selectedTheme);

            if (position < 0)
                this.lstThemes.Items.Add(selectedTheme);
        }

        /// <summary>
        /// Añade un usuario a la lista de usuarios
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void btn_addUser_Click(object sender, EventArgs e)
        {
            this.lst_users.ClearSelection();
            ListItem selectedUser = this.ddl_users.Items[this.ddl_users.SelectedIndex];

            //Solo añade el tema cuando no se encuentra en la lista
            int position = this.lst_users.Items.IndexOf(selectedUser);

            if (position < 0)
                this.lst_users.Items.Add(selectedUser);

        }

        /// <summary>
        /// Elimina de la lista el tema seleccionado
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void btn_deleteTheme_Click(object sender, EventArgs e)
        {
            int selectedIndex = this.lstThemes.SelectedIndex;
            if (selectedIndex >= 0)
                this.lstThemes.Items.RemoveAt(selectedIndex);
        }

        /// <summary>
        /// Elimina de la lista el usuario seleccionado
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void btn_deleteUser_Click(object sender, EventArgs e)
        {
            int selectedIndex = this.lst_users.SelectedIndex;
            if (selectedIndex >= 0)
                this.lst_users.Items.RemoveAt(selectedIndex);

        }
    }
}