USE [UOC_IDEAS]
GO

/****** Object:  Table [dbo].[Applications]    Script Date: 12/17/2011 20:36:50 ******/
IF  NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[Applications]') AND type in (N'U'))
BEGIN 
  PRINT 'Creating the Applications table...'
  CREATE TABLE [dbo].Applications (
    ApplicationName         nvarchar(235)               NOT NULL UNIQUE,
    ApplicationId           uniqueidentifier            PRIMARY KEY NONCLUSTERED DEFAULT NEWID(),
    Description             nvarchar(256)       )
  CREATE CLUSTERED INDEX Applications_Index ON [dbo].Applications(ApplicationName)
END
GO 


/****** Object:  Table [dbo].[Users]    Script Date: 12/17/2011 20:39:21 ******/
IF  NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[Users]') AND type in (N'U'))
BEGIN
  PRINT 'Creating the Users table...'
  CREATE TABLE [dbo].Users (
    ApplicationId    uniqueidentifier    NOT NULL FOREIGN KEY REFERENCES [dbo].Applications(ApplicationId),
    UserId           uniqueidentifier    NOT NULL PRIMARY KEY NONCLUSTERED DEFAULT NEWID(),
    UserName         nvarchar(50)       NOT NULL,
    IsAnonymous      bit                 NOT NULL DEFAULT 0,
    LastActivityDate DATETIME            NOT NULL)

   CREATE UNIQUE CLUSTERED INDEX Users_Index ON [dbo].Users(ApplicationId, UserName)
   CREATE NONCLUSTERED INDEX Users_Index2 ON [dbo].Users(ApplicationId, LastActivityDate)
END
GO


/****** Object:  Table [dbo].[Roles]    Script Date: 12/17/2011 20:41:58 ******/
IF  NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[Roles]') AND type in (N'U'))
BEGIN
  PRINT 'Creating the Roles table...'
  CREATE TABLE dbo.Roles (
        ApplicationId    uniqueidentifier    NOT NULL FOREIGN KEY REFERENCES dbo.Applications(ApplicationId),
        RoleId           uniqueidentifier    PRIMARY KEY  NONCLUSTERED DEFAULT NEWID(),
        RoleName         nvarchar(256)       NOT NULL,
        Description      nvarchar(256)       )
 CREATE UNIQUE  CLUSTERED  INDEX Roles_index1 ON  dbo.Roles(ApplicationId, RoleName)
END
GO


/****** Object:  Table [dbo].[UsersInRoles]    Script Date: 12/17/2011 20:43:21 ******/
IF  NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[UsersInRoles]') AND type in (N'U'))
BEGIN
  PRINT 'Creating the UsersInRoles table...'
  CREATE TABLE dbo.UsersInRoles (
        UserId     uniqueidentifier NOT NULL PRIMARY KEY(UserId, RoleId) FOREIGN KEY REFERENCES dbo.Users (UserId),
        RoleId     uniqueidentifier NOT NULL FOREIGN KEY REFERENCES dbo.Roles (RoleId))

  CREATE INDEX UsersInRoles_index ON  dbo.UsersInRoles(RoleId)	
END
GO

/****** Object:  Table [dbo].[Memberships]    Script Date: 12/17/2011 20:44:38 ******/
IF  NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[Memberships]') AND type in (N'U'))
BEGIN 
  PRINT 'Creating the Membership table...'
  CREATE TABLE dbo.Memberships (
        ApplicationId                           uniqueidentifier    NOT NULL FOREIGN KEY REFERENCES dbo.Applications(ApplicationId),
        UserId                                  uniqueidentifier    NOT NULL PRIMARY KEY NONCLUSTERED FOREIGN KEY REFERENCES dbo.Users(UserId),
        Password                                nvarchar(128)       NOT NULL,
        PasswordFormat                          int                 NOT NULL DEFAULT 0,
        PasswordSalt                            nvarchar(128)       NOT NULL,
        Email                                   nvarchar(256),
        PasswordQuestion                        nvarchar(256),
        PasswordAnswer                          nvarchar(128),
        IsApproved                              bit                 NOT NULL,
        IsLockedOut                             bit                 NOT NULL,
        CreateDate                              datetime            NOT NULL,
        LastLoginDate                           datetime            NOT NULL,
        LastPasswordChangedDate                 datetime            NOT NULL,
        LastLockoutDate                         datetime            NOT NULL,
        FailedPasswordAttemptCount              int                 NOT NULL,
        FailedPasswordAttemptWindowStart        datetime            NOT NULL,
        FailedPasswordAnswerAttemptCount        int                 NOT NULL,
        FailedPasswordAnswerAttemptWindowStart  datetime            NOT NULL,
        Comment                                 nvarchar(256) )
  CREATE CLUSTERED INDEX Memberships_index ON Memberships(ApplicationId, Email)
END
GO

/****** Object:  Table [dbo].[Profiles]    Script Date: 12/17/2011 20:48:12 ******/
IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[Profiles]') AND type in (N'U'))
BEGIN

  PRINT 'Creating the Profile table...'
  CREATE TABLE dbo.Profile (
        UserId                   uniqueidentifier  PRIMARY KEY FOREIGN KEY REFERENCES dbo.Users(UserId),
        PropertyNames            nvarchar(4000) NOT NULL,
        PropertyValuesString     nvarchar(4000) NOT NULL,
        PropertyValuesBinary     image NOT NULL,
        LastUpdatedDate          datetime NOT NULL)
END
GO











