-- TABLA DE SISTEMA. PAISES

  CREATE TABLE [dbo].tbl_sys_paises (
    pk_uid_id			uniqueidentifier               PRIMARY KEY NONCLUSTERED DEFAULT NEWID(),
    str_codigo			nvarchar(8)					   NOT NULL UNIQUE,
    str_nombre          nvarchar(64)                   NOT NULL UNIQUE)
    CREATE CLUSTERED INDEX tbl_sys_paises_Index ON [dbo].tbl_sys_paises(str_nombre)
GO

-- ORGANIZACIONES
  CREATE TABLE [dbo].tbl_organizaciones (
    pk_uid_id			uniqueidentifier               PRIMARY KEY NONCLUSTERED DEFAULT NEWID(),
    str_codigo			nvarchar(8)					   NOT NULL UNIQUE,
    str_nombre          nvarchar(128)                  NOT NULL UNIQUE,
    str_cif				nvarchar(16)				   DEFAULT NULL,
    str_direccion		nvarchar(256)				   DEFAULT NULL,
    str_cp				nvarchar(8)					   DEFAULT NULL,
    str_poblacion		nvarchar(128)				   DEFAULT NULL,
    str_provincia		nvarchar(64)				   DEFAULT NULL,
    str_pais			nvarchar(64)				   DEFAULT NULL,
    str_telefono		nvarchar(16)				   DEFAULT NULL,
    str_fax				nvarchar(16)				   DEFAULT NULL	
    )
    CREATE CLUSTERED INDEX tbl_organizaciones_Index ON [dbo].tbl_organizaciones(str_nombre)
GO

--PERSONALIZACIN DE USUARIOS
  CREATE TABLE [dbo].tbl_personalizacionUsuarios (
    pk_uid_id			uniqueidentifier               PRIMARY KEY NONCLUSTERED,
    uid_aplicacion		uniqueidentifier			   NOT NULL,
    str_nombre          nvarchar(64)                   DEFAULT NULL,
    str_apellidos		nvarchar(128)				   DEFAULT NULL,
    str_telefono		nvarchar(16)				   DEFAULT NULL,
    uid_organizacion	uniqueidentifier			   DEFAULT NULL
    )
    CREATE CLUSTERED INDEX tbl_personalizacionUsuarios_Index ON [dbo].tbl_personalizacionUsuarios(str_nombre)
GO

-- FOREIGN KEY a organizaciones
ALTER TABLE [dbo].tbl_personalizacionUsuarios  WITH CHECK ADD  CONSTRAINT [UsuariosOrganizaciones] FOREIGN KEY([uid_organizacion])
REFERENCES [dbo].tbl_organizaciones ([pk_uid_id])
GO

--FOREIGN KEY a usuarios y aplicaciones se controlar por cdigo

--TEMAS
  CREATE TABLE [dbo].tbl_temas (
    pk_uid_id			uniqueidentifier               PRIMARY KEY NONCLUSTERED,
    str_codigo			nvarchar(8)					   NOT NULL UNIQUE,
    str_nombre          nvarchar(128)                  NOT NULL UNIQUE,
    bol_activo			bit							   DEFAULT 0
    )
    CREATE CLUSTERED INDEX tbl_temas_Index ON [dbo].tbl_temas(str_nombre)
GO

--RAZN DE ARCHIVO
CREATE TABLE [dbo].tbl_sys_razonArchivo (
    pk_lng_id			bigint						   PRIMARY KEY NONCLUSTERED,
    str_codigo			nvarchar(8)					   NOT NULL UNIQUE,
    str_nombre          nvarchar(64)                   NOT NULL UNIQUE
    )
    CREATE CLUSTERED INDEX tbl_razonArchivo_Index ON [dbo].tbl_sys_razonArchivo(str_nombre)
GO

--ESTADO IDEA
  CREATE TABLE [dbo].tbl_sys_estadoIdea (
    pk_lng_id			bigint							PRIMARY KEY NONCLUSTERED,
    str_codigo			nvarchar(8)					   NOT NULL UNIQUE,
    str_nombre          nvarchar(64)                   NOT NULL UNIQUE
    )
    CREATE CLUSTERED INDEX tbl_estadoIdea_Index ON [dbo].tbl_sys_estadoIdea(str_nombre)
GO


--IDEAS
  CREATE TABLE [dbo].tbl_ideas (
    pk_uid_id				uniqueidentifier               PRIMARY KEY NONCLUSTERED DEFAULT NEWID(),
    str_titulo				nvarchar(64)				   NOT NULL UNIQUE,
    str_descripcion			nvarchar(512)                  DEFAULT NULL,
    dat_fechaCreacion		datetime					   NOT NULL,
    dat_fechalimite			datetime					   NOT NULL,
    epk_lng_idEstado		bigint						   NOT NULL,
    bol_archivada			bit							   DEFAULT 0,
    epk_lng_razonArchivo	bigint						   NULL,
    epk_uid_propietario		uniqueidentifier			   NOT NULL
    )
    CREATE CLUSTERED INDEX tbl_ideas_Index ON [dbo].tbl_ideas(str_titulo)
GO

-- FOREIGN KEY a estado idea
ALTER TABLE [dbo].tbl_ideas  WITH CHECK ADD  CONSTRAINT [estadoIdea] FOREIGN KEY([epk_lng_idEstado])
REFERENCES [dbo].tbl_sys_estadoIdea ([pk_lng_id])
GO

-- FOREIGN KEY a razonArchivo
ALTER TABLE [dbo].tbl_ideas  WITH CHECK ADD  CONSTRAINT [razonArchivo] FOREIGN KEY([epk_lng_razonArchivo])
REFERENCES [dbo].tbl_sys_razonArchivo ([pk_lng_id])
GO

-- La foreign key al propietario ser lgica

--Asociacin de IDEAS y TEMAS
  CREATE TABLE [dbo].tbl_rel_ideatema (
    pk_uid_id				uniqueidentifier               PRIMARY KEY NONCLUSTERED DEFAULT NEWID(),
    epk_uid_idea			uniqueidentifier			   NOT NULL,
    epk_uid_tema			uniqueidentifier               NOT NULL
    )
    CREATE CLUSTERED INDEX tbl_rel_ideatema_Index ON [dbo].tbl_rel_ideatema(epk_uid_idea,epk_uid_tema)
GO

-- FOREIGN KEY a Idea
ALTER TABLE [dbo].tbl_rel_ideatema  WITH CHECK ADD  CONSTRAINT [ideaTemaIdea] FOREIGN KEY([epk_uid_idea])
REFERENCES [dbo].tbl_ideas ([pk_uid_id])
GO

-- FOREIGN KEY a Tema
ALTER TABLE [dbo].tbl_rel_ideatema  WITH CHECK ADD  CONSTRAINT [ideaTemaTema] FOREIGN KEY([epk_uid_tema])
REFERENCES [dbo].tbl_temas ([pk_uid_id])
GO



--Asociacin de IDEAS y USUARIOS
CREATE TABLE [dbo].tbl_rel_ideausuario (
    pk_uid_id				uniqueidentifier               PRIMARY KEY NONCLUSTERED DEFAULT NEWID(),
    epk_uid_idea			uniqueidentifier			   NOT NULL,
    epk_uid_usuario			uniqueidentifier               NOT NULL
    )
    CREATE CLUSTERED INDEX tbl_rel_ideausuario_Index ON [dbo].tbl_rel_ideausuario(epk_uid_idea,epk_uid_usuario)
GO

-- FOREIGN KEY a Idea
ALTER TABLE [dbo].tbl_rel_ideausuario  WITH CHECK ADD  CONSTRAINT [ideaUsuarioIdea] FOREIGN KEY([epk_uid_idea])
REFERENCES [dbo].tbl_ideas ([pk_uid_id])
GO

-- FOREIGN KEY a Usuario, relacin logica

-- COMENTARIOS
  CREATE TABLE [dbo].tbl_comentarios (
    pk_uid_id				uniqueidentifier               PRIMARY KEY NONCLUSTERED DEFAULT NEWID(),
    dat_fechaComentario		datetime					   NOT NULL,
    str_comentario			nvarchar(450)				   NOT NULL,
    epk_uid_idea			uniqueidentifier               NOT NULL,
    epk_uid_usuario			uniqueidentifier			   NOT NULL
    )
    CREATE CLUSTERED INDEX tbl_comentarios_Index ON [dbo].tbl_comentarios(str_comentario)
GO

-- FOREIGN KEY a Idea
ALTER TABLE [dbo].tbl_comentarios  WITH CHECK ADD  CONSTRAINT [comentarioIdea] FOREIGN KEY([epk_uid_idea])
REFERENCES [dbo].tbl_ideas ([pk_uid_id])
GO

-- FOREIGN KEY a Usuario, relacin logica

-- VALORACION
  CREATE TABLE [dbo].tbl_valoracion (
    pk_uid_id				uniqueidentifier               PRIMARY KEY NONCLUSTERED DEFAULT NEWID(),
    dat_fechaValoracion		datetime					   NOT NULL,
    int_valoracion			int							   NOT NULL,
    epk_uid_idea			uniqueidentifier               NOT NULL,
    epk_uid_usuario			uniqueidentifier			   NOT NULL
    )
    CREATE CLUSTERED INDEX tbl_valoracion_Index ON [dbo].tbl_valoracion(int_valoracion)
GO

-- FOREIGN KEY a Idea
ALTER TABLE [dbo].tbl_valoracion  WITH CHECK ADD  CONSTRAINT [valoracionIdea] FOREIGN KEY([epk_uid_idea])
REFERENCES [dbo].tbl_ideas ([pk_uid_id])
GO

-- FOREIGN KEY a Usuario, relacin logica

